package repository

import (
	"time"

	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/entity"
	"gorm.io/gorm"
)

type BatchSeminarRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewBatchSeminarRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *BatchSeminarRepo {
	return &BatchSeminarRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *BatchSeminarRepo) NewBatchSeminar(batchseminar entity.BatchSeminar) (output entity.BatchSeminar, err error) {
	batchseminar.IdBatch = uuid.New().String()
	batchseminar.Status = true

	err = repo.DBWrite.Model(&entity.BatchSeminar{}).Where("status = ?", true).Update("status", false).Error
	if err != nil {
		return entity.BatchSeminar{}, err
	}

	result := repo.DBWrite.Create(&batchseminar)

	if result.Error != nil {
		return entity.BatchSeminar{}, result.Error
	}
	return batchseminar, nil
}

func (repo *BatchSeminarRepo) GetSeminarBatches() (output []entity.BatchSeminar, err error) {
	var batches []entity.BatchSeminar
	result := repo.DBRead.Find(&batches)
	if result.Error != nil {
		return nil, err
	}
	return batches, nil
}

func (repo *BatchSeminarRepo) DeleteBatchSeminar(id string) (entity.BatchSeminar, error) {
	batch := entity.BatchSeminar{IdBatch: id}
	err := repo.DBWrite.Delete(&batch).Error

	if err != nil {
		return batch, err
	}

	return batch, err
}

func (repo *BatchSeminarRepo) GetBatchSeminarCurrent() ([]entity.BatchSeminar, error) {
	var batch []entity.BatchSeminar
	currentTime := time.Now().Format("2006-01-02T15:04:05Z")

	result := repo.DBRead.Where("? BETWEEN start_date AND end_date", currentTime).Limit(1).Find(&batch)
	if result.Error != nil {
		return nil, result.Error
	}

	return batch, nil
}

func (repo *BatchSeminarRepo) SetLiveBatch(id string) (entity.BatchSeminar, error) {
	err := repo.DBWrite.Model(&entity.BatchSeminar{}).Where("status = ?", true).Update("status", false).Error
	if err != nil {
		return entity.BatchSeminar{}, err
	}

	batch := entity.BatchSeminar{IdBatch: id}
	err = repo.DBWrite.Model(&batch).Update("status", true).Error

	if err != nil {
		return batch, err
	}

	return batch, nil
}

func (repo *BatchSeminarRepo) GetBatchSeminarLive() ([]entity.BatchSeminar, error) {
	var batch []entity.BatchSeminar

	result := repo.DBRead.Where("status = ?", true).Limit(1).Find(&batch)
	if result.Error != nil {
		return nil, result.Error
	}

	return batch, nil
}
