package usecase

import (
	"encoding/json"
	"errors"
	"io"
	"net/http"
	"os"

	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_tim_tugas/entity"
)

type DashboardTimTugasUc struct {
	dashboardTimTugasRepo DashboardTimTugasRepo
}

func NewDashboardTimTugasUc(dashboardTimTugasRepo DashboardTimTugasRepo) *DashboardTimTugasUc {
	return &DashboardTimTugasUc{
		dashboardTimTugasRepo: dashboardTimTugasRepo,
	}
}

func (uc *DashboardTimTugasUc) GetDashboardStatistics() (entity.StatusStatistics, error) {
	url := os.Getenv("S1_URL")
	res, reqErr := http.Get(url + "/api/internal/batch-seminar-live")
	if reqErr != nil {
		return entity.StatusStatistics{}, reqErr
	}
	defer res.Body.Close()

	body, parseErr := io.ReadAll(res.Body)
	if parseErr != nil {
		return entity.StatusStatistics{}, parseErr
	}

	var parsedRes map[string]interface{}
	parsedErr := json.Unmarshal(body, &parsedRes)
	if parsedErr != nil {
		return entity.StatusStatistics{}, parsedErr
	}

	var id string
	if parsedRes["id"] != nil {
		id, _ = parsedRes["id"].(string)
		if id == "" {
			return entity.StatusStatistics{}, errors.New("no active period")
		}
	} else {
		id = "00000000-0000-0000-0000-000000000000"
	}

	return uc.dashboardTimTugasRepo.GetDashboardStatistics(id)
}

func (uc *DashboardTimTugasUc) GetDaftarMahasiswa() ([]entity.DataPendaftaranMahasiswa, error) {
	url := os.Getenv("S1_URL")
	res, reqErr := http.Get(url + "/api/internal/batch-seminar-live")
	if reqErr != nil {
		return nil, reqErr
	}
	defer res.Body.Close()

	body, parseErr := io.ReadAll(res.Body)
	if parseErr != nil {
		return nil, parseErr
	}

	var parsedRes map[string]interface{}
	parsedErr := json.Unmarshal(body, &parsedRes)
	if parsedErr != nil {
		return nil, parsedErr
	}

	var id string
	if parsedRes["id"] != nil {
		id, _ = parsedRes["id"].(string)
		if id == "" {
			return nil, errors.New("no active period")
		}
	} else {
		id = "00000000-0000-0000-0000-000000000000"
	}

	return uc.dashboardTimTugasRepo.GetDaftarMahasiswa(id)
}

func (uc *DashboardTimTugasUc) GetPeriodData() ([]entity.PeriodStats, error) {
	return uc.dashboardTimTugasRepo.GetPeriodData()
}

// func (uc *DashboardTimTugasUc) GetStatsDaftarMahasiswa() ([]entity.StatMahasiswa, error) {
// 	return uc.dashboardTimTugasRepo.GetStatsDaftarMahasiswa()
// }