package usecase

import (
	"fmt"

	"gitlab.informatika.org/k-01-11/graduit-be/src/module/scheduling/entity"
)

type SchedulingUsecase interface {
	AddSchedulingLink(idAndLink entity.SchedulingLink) error
	GetSchedulingTableData() (tableData []entity.JadwalWithLink, err error)
}

type SchedulingUc struct {
	schedulingRepo SchedulingRepository
}

func NewSchedulingUseCase(schedulingRepo SchedulingRepository) *SchedulingUc {
	return &SchedulingUc{
		schedulingRepo: schedulingRepo,
	}
}

func (uc *SchedulingUc) AddSchedulingLink(idAndLink entity.SchedulingLink) error {
	if idAndLink.SidangOrSeminar == "sidang" {
		sidangLink := entity.SchedulingSidangLink{
			ID:   idAndLink.ID,
			Link: idAndLink.Link,
		}
		err := uc.schedulingRepo.NewSchedulingLinkSidang(sidangLink)
		if err != nil {
			return err
		}
		return nil
	} else if idAndLink.SidangOrSeminar == "seminar" {
		seminarLink := entity.SchedulingSeminarLink{
			ID:   idAndLink.ID,
			Link: idAndLink.Link,
		}
		err := uc.schedulingRepo.NewSchedulingLinkSeminar(seminarLink)
		if err != nil {
			return err
		}
		return nil
	}
	return fmt.Errorf("bad request: type isnt sidang nor seminar")
}

func (uc *SchedulingUc) GetSchedulingTableData() (tableData []entity.JadwalWithLink, err error) {
	sidangResult, seminarResult, err := uc.schedulingRepo.GetSchedulingTableData()
	if err != nil {
		return []entity.JadwalWithLink{}, err
	}

	for i := 0; i < len(sidangResult); i++ {
		newRow := entity.JadwalWithLink{
			ID:              sidangResult[i].ID,
			Nim:             sidangResult[i].Nim,
			Name:            sidangResult[i].Name,
			NamaDosbing:     sidangResult[i].NamaDosbing,
			NamaDosuji1:     sidangResult[i].NamaDosuji1,
			NamaDosuji2:     sidangResult[i].NamaDosuji2,
			Ruang:           sidangResult[i].Ruang,
			IDBatch:         sidangResult[i].IDBatch,
			Judul:           sidangResult[i].Judul,
			SidangOrSeminar: "Sidang",
			Link:            sidangResult[i].Link,
			ScheduledAt:     sidangResult[i].ScheduledAt,
		}
		tableData = append(tableData, newRow)
	}

	for i := 0; i < len(seminarResult); i++ {
		newRow := entity.JadwalWithLink{
			ID:              seminarResult[i].ID,
			Nim:             seminarResult[i].Nim,
			Name:            seminarResult[i].Name,
			NamaDosbing:     seminarResult[i].NamaDosbing,
			NamaDosuji1:     seminarResult[i].NamaDosuji1,
			NamaDosuji2:     seminarResult[i].NamaDosuji2,
			Ruang:           seminarResult[i].Ruang,
			IDBatch:         seminarResult[i].IDBatch,
			Judul:           seminarResult[i].Judul,
			SidangOrSeminar: "Seminar",
			Link:            seminarResult[i].Link,
			ScheduledAt:     seminarResult[i].ScheduledAt,
		}
		tableData = append(tableData, newRow)
	}

	return tableData, nil
}
